package org.alltimeflashdreamer.log4jmonitor;

import com.incors.plaf.kunststoff.KunststoffLookAndFeel;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.ImageProducer;
import java.io.*;
import java.net.URL;
import java.util.jar.JarFile;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.metal.MetalLookAndFeel;
import org.apache.log4j.*;
import org.apache.log4j.spi.LoggingEvent;

/**
 * Title: Log4JMonitor<br>
 * Description: This is the application window and the main class of the Log4JMonitor application.<br>
 * It serves as a SocketServer for the Log4J remote logging service and is therefore able to show
 * Log4J messages from somewhere in the network that sends such messages to the localhost.<br>
 * The received messages are listed in the application's JTable and rendered according to their
 * relevance and type.
 *
 * @author    Copyright (c) 2001 by Sven Mosimann, Adrian Seiler, Florian Steinsiepe of EcoLogic GmbH,
 *            revised 2002 by <a href="mailto:florian@steinsiepe.ch">Florian Steinsiepe</a>
 * @version   1.1
 */

public class Log4JMonitor extends JFrame implements WindowListener, ActionListener
{
    //constants
	/** the version number of the application */
	static final String VERSION = "1.1";
    /** the title of the application, shown in the title bar of the main window */
    static final String TITLE = "Log4JMonitor";
    /** the "adjust (table) column widths" selection text */
    private static final String ADJUSTCOLUMNWIDTHS = "Adjust column widths";
    /** the "exit program" selection text */
    private static final String EXIT = "Exit";
    /** the "file menu" text */
    private static final String FILE = "File";
	/** the "clear event table" text */
    private static final String CLEAR = "Clear";
	/** the "pause receiving events" text */
    private static final String PAUSE = "Pause/Resume";
	/** the "show about dialog" text */
	private static final String ABOUT = "About...";
	/** the "actions menu" text */
	private static final String ACTIONS = "Actions";

    private static final int SPACEHEIGHT = 1;
    private static final int SPACEWIDTH = 15;
    private static final int DEFAULTWINDOWHEIGHT = 600;
    private static final int DEFAULTWINDOWWIDTH = 800;
    private static final int LOGOPANELBORDERTHICKNESS = 10;
    private static final Level LEVEL_DEFAULT = Level.DEBUG;

    //GUI components
    private JMenuItem jmiExit = new JMenuItem( EXIT, KeyEvent.VK_E );
    private JMenuItem jmiAdjustColumnWidths = new JMenuItem( ADJUSTCOLUMNWIDTHS, KeyEvent.VK_C );
	private JMenuItem jmiShowAboutDialog = new JMenuItem( ABOUT, KeyEvent.VK_B );
	private JMenuItem jmiClearTable = new JMenuItem( CLEAR, KeyEvent.VK_L );
	private JMenuItem jmiPauseReceiving = new JMenuItem( PAUSE, KeyEvent.VK_P );
    private JMenu jmMenuFile = new JMenu( FILE );
	private JMenu jmMenuActions = new JMenu( ACTIONS );
    private JMenuBar jmbMenu = new JMenuBar();
    private JPanel jpSpaceEAST = new JPanel();
    private JPanel jpSpaceWEST = new JPanel();
    private JPanel jpLogoPanel = new JPanel( new BorderLayout() );
    private LogPanel logPanel;
    private PatternLayout layout;
    private Color colorBackground;
    private Border borderDefault;
    private Border borderLogos;
	private static int iWindowWidth = DEFAULTWINDOWWIDTH;
	private static int iWindowHeight = DEFAULTWINDOWHEIGHT;
    private static Dimension inititalWindowSize = new Dimension( iWindowWidth, iWindowHeight );


    /**
     * The constructor initializes the Log4J and some often used GUI components and
     * calls the buildGUI method, catching possibly upcoming exceptions.<br>
     * Finally, a Thread is started which accepts SocketConnections (on port 27272
	 * by default, if not overwritten by setting the JVM's "log4jmonitor.port"
	 * property), sent by a SocketAppender (as defined in the related log4j.properties file).
     *
     * @exception Exception  wrapper for any possible exception that may occur
	 *            and isn't handled within the method itself
	 * @see java.net.ServerSocket
	 * @see org.apache.log4j.net.SocketAppender
     */
    public Log4JMonitor()
        throws Exception
    {
        //set frame icon
		this.setIconImage( getImageFromFileOrFromJar( "Log4JMonitor.gif" ) );

        //use frame's background color for each component of the GUI
        colorBackground = this.getBackground();

        //create a default line border to be used by each "bordered" component
        borderDefault = BorderFactory.createLineBorder( colorBackground, LOGOPANELBORDERTHICKNESS );
        borderLogos = BorderFactory.createLineBorder( Color.white, LOGOPANELBORDERTHICKNESS );

        //set's the log output layout/style
        layout = new PatternLayout( "%d: %-5p - %m%n" );

        //initialize the panel where the log messages are shown in
        logPanel = new LogPanel();

        buildGUI();

        //finally, show itself
        this.show();
    }

    /**
     * The main method starts the control monitor.
     *
     * @param args  the command line arguments
     */
    public static void main( String[] args )
    {
        try
        {
			//set the Look'n'Feel
            UIManager.setLookAndFeel( new KunststoffLookAndFeel() );
        } catch ( UnsupportedLookAndFeelException ulfe )
        {
            try
            {
                UIManager.setLookAndFeel( new MetalLookAndFeel() );
            } catch ( UnsupportedLookAndFeelException ulfe2 )
            {
                ulfe2.printStackTrace();
            }
		}

		//set the window size, if defined in the command line arguments (first
		//argument: window width, second argument: window size)
		if ( args != null && args.length > 0 )
		{
			try  {
				inititalWindowSize = new Dimension(
				    new Integer( args[0] ).intValue(), new Integer( args[1] ).intValue() );
			} catch ( Exception ex ) {
				printUsage();
			}
		}

		//start app
		try {
            new Log4JMonitor();
        } catch ( Exception e )
        {
            e.printStackTrace();
        }
    }

//---------------- WindowListener methods ---------------------------//
    /**
     * Exits the virtual machine silently.
     *
     * @param event  the window event to handle
     */
    public void windowClosing( WindowEvent event )
    {
        System.exit( 0 );
    }

    /**
     * Sets the table's column widths accordingly.
     *
     * @param event  the window event to handle
     */
    public void windowOpened( WindowEvent event )
	{
        logPanel.setColumnWidths();
	}

    /**
     * (inactive)
     *
     * @param event  the window event to handle
     */
    public void windowClosed( WindowEvent event ) { }

    /**
     * (inactive)
     *
     * @param event  the window event to handle
     */
    public void windowIconified( WindowEvent event ) {}

    /**
     * (inactive)
     *
     * @param event  the window event to handle
     */
    public void windowDeiconified( WindowEvent event ) { }

    /**
     * (inactive)
     *
     * @param event  the window event to handle
     */
    public void windowActivated( WindowEvent event ) {}

    /**
     * (inactive)
     *
     * @param event  the window event to handle
     */
    public void windowDeactivated( WindowEvent event ) { }

//---------------- ActionListener method ---------------------------//
    /**
     * The actionPerformed method handles the action events that come up by user
     * actions. The possible sources are those listed in the menu(s).
     *
     * @param event  the action event to handle
     */
    public void actionPerformed( ActionEvent event )
    {
        if ( event.getActionCommand().equals( EXIT ) )
            System.exit( 0 );
		else if ( event.getActionCommand().equals( CLEAR ) )
		    logPanel.cp.clearButton.doClick();
		else if ( event.getActionCommand().equals( PAUSE ) )
		    logPanel.cp.toggleButton.doClick();
		else if ( event.getActionCommand().equals( ADJUSTCOLUMNWIDTHS ) )
            logPanel.setColumnWidths();
		else if ( event.getActionCommand().equals( ABOUT ) )
		    new AboutDialog( this );
    }

//---------------- internal method ---------------------------//
    /**
     * Puts the GUI components together, forming the control manager's window.
     *
     * @exception Exception  wrapper for any exception that may occur
     */
    private void buildGUI()
        throws Exception
    {
        this.setBackground( colorBackground );
        this.addWindowListener( this );

        //put window components together into the frame
        this.setSize( inititalWindowSize );
        this.setJMenuBar( jmbMenu );
        this.setTitle( TITLE );
        this.getContentPane().setLayout( new BorderLayout() );

        //add menues to menu bar
        jmbMenu.add( jmMenuFile );
		jmbMenu.add( jmMenuActions );

        //set ShortCuts
        jmMenuFile.setMnemonic( KeyEvent.VK_F );
		jmMenuActions.setMnemonic( KeyEvent.VK_A );

        //fill in menu items
        jmMenuFile.add( jmiExit );
		jmMenuActions.add( jmiClearTable );
		jmMenuActions.add( jmiPauseReceiving );
		jmMenuActions.addSeparator();
		jmMenuActions.add( jmiAdjustColumnWidths );
		jmMenuActions.addSeparator();
		jmMenuActions.add( jmiShowAboutDialog );

        //set action listeners to menu items as well as key strokes
        jmiExit.addActionListener( this );
        jmiExit.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_F4, KeyEvent.ALT_MASK ) );
		jmiAdjustColumnWidths.addActionListener( this );
        jmiAdjustColumnWidths.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_C, KeyEvent.ALT_MASK ) );
		jmiShowAboutDialog.addActionListener( this );
		jmiShowAboutDialog.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_B, KeyEvent.ALT_MASK ) );
		jmiClearTable.addActionListener( this );
		jmiClearTable.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_L, KeyEvent.ALT_MASK ) );
		jmiPauseReceiving.addActionListener( this );
		jmiPauseReceiving.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_P, KeyEvent.ALT_MASK ) );

		//set border, sizes and backgrounds
        jpLogoPanel.setBorder( borderDefault );
        jpLogoPanel.setBackground( Color.white );
        jpSpaceWEST.setSize( SPACEHEIGHT, SPACEWIDTH );
        jpSpaceWEST.setBackground( colorBackground );
        jpSpaceEAST.setSize( SPACEHEIGHT, SPACEWIDTH );
        jpSpaceEAST.setBackground( colorBackground );

        //set content pane components
        this.getContentPane().add( logPanel, BorderLayout.CENTER );
        this.getContentPane().add( jpLogoPanel, BorderLayout.SOUTH );
        this.getContentPane().add( jpSpaceWEST, BorderLayout.WEST );
        this.getContentPane().add( jpSpaceEAST, BorderLayout.EAST );

		//calculate and set ideal position
		Dimension origin = calculateOrigin( this.getSize() );
		this.setLocation( origin.width, origin.height );
    }

    /**
     * Makes an often needed job: Calculates the origin of the dialog to be shown.
     *
     * @param sizeOfDialog  the size of the currently shown window
     * @return              the origin that positions this window in th center of
	 *                      the screen
     */
    private Dimension calculateOrigin( Dimension sizeOfDialog )
    {
        int x = this.getX() + ( ( this.getWidth() > 0 ) ? this.getWidth() / 2 : iWindowWidth ) - sizeOfDialog.width / 2;
        int y = this.getY() + ( ( this.getHeight() > 0 ) ? this.getHeight() / 2 : iWindowHeight ) - sizeOfDialog.height / 2;
        return new Dimension( x, y );
    }

	/**
	 * Prints out the usage text.
	 */
	private static void printUsage()
	{
		System.out.println( "Usage of Log4JMonitor's command line call is:" );
		System.out.println( "java -classpath \".;Log4JMonitor.jar\" org.alltimeflashdreamer.log4jmonitor.Log4JMonitor [window-width window-height]" );
		System.out.println( "Explanation: If started with command line arguments, the first one is interpreted as an int " +
							"specifying the window width and the second one specifying the window height (both values in pixels). " +
							"If started without command line arguments, the window size is 800x600 pixels." );
	}

	/**
	 * Tries loading the given image from the package directory within the classes folder
	 * (extracted class files). If this fails, the image is tried to be loaded from the
	 * jar file named log4jmonitor-<version-number>.jar. If this fails too, null is returned
	 * else the image as a java.awt.Image.
	 *
	 * @param filename  the name of the image to be loaded (relative to the package folder)
	 * @return          the successfully loaded image or null
	 */
	private Image getImageFromFileOrFromJar( String filename )
	{
		try {
			InputStream in = getClass().getResourceAsStream( filename );
			if ( in == null )
			{
				JarFile jarFile = new JarFile( "./log4jmonitor-" + VERSION + ".jar" );
				in = jarFile.getInputStream( jarFile.getJarEntry( "org/alltimeflashdreamer/log4jmonitor/Log4JMonitor.gif" ) );
			}

			int length = in.available();
			byte[] byteInput = new byte[length];
			in.read( byteInput );
			return Toolkit.getDefaultToolkit().createImage( byteInput );

		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
		return null;
	}
}
